#!/usr/bin/perl

use File::stat;
use Time::localtime;
use POSIX qw(strftime);
use Digest::CRC qw(crc16);

# Created by Konstantin Kuzov (2:5019/40)
# Used part form PntCheck by Evgeny Vetrov (2:6037/7)
# Distributed under GPLv3

my $makemode    = ($ARGV[0] ? 1 : 0);
my $out_error	= 0 if (!defined $out_error);
my $out_format	= 1 if (!defined $out_format);	# 1 - boss; (2 - point; 3 - fakenet)?
my $pntseg	= "PNT5019.NDL" if (!defined $pntseg);
my $myzone      = "2" if (!defined $myzone);
my $mynet       = "5019" if (!defined $mynet);
my $segpath     = "/fido/pntlist/segments";
my $pntlist     = "/fido/pntlist/PNT$mynet.NDL";
#my $reportfile  = "/fido/pntlist/pointlist.report";
#my $reportecho  = "";
my $reportecho  = "ganjanet.tech";
my $reportfile  = "";

my $password = "PASSWORD";
my $robotname = "PNT$mynet.NDL Maker";
my $pktfrom = "2:5019/40";
my $pktto = "2:5019/40";
my $addrfrom = "2:5019/40";
my $tearline = "PNT$mynet.NDL Maker on GaNJaNET";
my $origin = "GaNJaNET STaTi0N";
my $mail_program = "/fido/hpt/txt2pkt -c /fido/hpt/config -p $password -xf $pktfrom -xt $pktto -af $addrfrom -nf '$robotname' -nt 'TONAME' -at 'TOADDR' -e 'TARGET' -t '$tearline' -o '$origin' -s 'SUBJECT' -d /fido/inbound -";

my $nlpath	= "/fido/nodelist" if (!defined $nlpath);
my $nlfile = <$nlpath*/NODELIST.???>;

open(NODELIST,"<",$nlfile) || die "Error opening nodelist :-(\n";
if ($makemode) {
    open POINTLIST, ">$pntlist" || die "Error creating pointlist :(\n";
}
my @nodes,@status,@sysopname;
while(<NODELIST>){
    next if(/^;/);
#    if(/^[^\w,]/) { print "Warning! Illegal line:\n", $_;};

    if(/^Zone,(\d+),/){
      $zone=$1;
      $net=$1;
      $node=0;
    }
    elsif(/^(Region|Host),(\d+),/){ # net/independent number
      $net=$2;
      $node=0;
    }
    elsif(/^(Hold),(\d+)/){      # skip nodes with Down/Hold status.
      $node=$2;
      if ($net eq $mynet) {
		@status[$node]=' Hold ';
		push(@nodes,"$node");
      }
      next;
    }
    elsif(/^(Down),(\d+)/){      # skip nodes with Down/Hold status.
      $node=$2;
      if ($net eq $mynet) {
		@status[$node]=' Down ';
		push(@nodes,$node);
      }
      next;
    }
    elsif(/^(Hub|Pvt|),(\d+),.*?,.*?,(.*?),/){ # node number
      $node=$2;
      $name=$3;
      $name =~ s/_/ /g;
    }
    else{
      next;
    }
    if ($zone eq $myzone && $net eq $mynet && $node ne 0) {
	@status[$node]='Normal';
	@sysopname[$node]=$name;
	push(@nodes,"$node");
	next;
    }
}
close(NODELIST);

sub addingnull
{
        my($string,$count)=@_;
        for ($i=length($string); $i<$count;$i++)
        {
                $string="0".$string;
        }
        return $string;
}

sub find_node {
	local($addr)=@_;
	my ($z,$n,$f)=split(/:|\//,$addr);
	open(NL,"<",$nlfile);
	while(<NL>) {
		last if /^Zone,$z/i;
	}
	while(<NL>) {
		last if /^(Region|Host),$n/i;
	}
	if ($f eq "0") {
		return $_;
	}
	while(<NL>) {
		last if /^(Zone|Region|Host),/i;
		if ( /^(Hub|Pvt|Hold|Down)?,$f,/i ) {
			close(NL);
			return $_;
		}
	}
	close(NL);
	return "123";
}

my $segstatus;

sub check_plist {
	local($file,$net,$format,$error,$node)=@_;
	my @valid_speed=(
"300", "1200", "2400", "4800", "9600", "14400", "16800", "19200", "28800", "33600"
);
	my @valid_flags=(
"CM","ICM","MO","LO","MN",
"V22","V29","V32","V32(b|B)","V34","V90(c|C)","V90(s|S)",
"V32T","VFC",
"HST","H14","H16","X2C","X2S","ZYX","Z19","H96","PEP","CSP",
"MNP","V42","V42(b|B)",
"XA","XB","XC","XP","XR","XW","XX",
"T[A-Xa-x]{2}",
"V110L","V110H","V120L","V120H","X75","ISDN",
"(INA|IP|IBN|IFC|IFT|ITN|IVM)(:[A-Za-z1-9\.-_]+)?(:[0-9]+)?",
"(IEM|ITX|IUC|IMI|ISE):[A-Za-z1-9\.-_@]+",
"PING"
);
	my @valid_user_flags=(
"ZEC","REC","NEC","NC","SDS","SMH","RPK","NPK","ENC","CDP","T[A-Xa-x]{2}"
);
	my $in_format = 0;
	my $ret = "";
	my $z = 2, $n = 0, $f = 0;
	open(PLIST,$file);
	@e = ();
	$segstatus='';
	$critical=0;
	$fixed = 0;
	while(<PLIST>) {
		my $r = "";
		if ($_=~/\A\;(S|U|F|A|E)*([^\r\n]*)/) {
			$ret.=";$1$2\r\n";
		} else {
			my ($kw,$point,$sysname,$location,$name,$phone,$speed,@flags) = split(/,|\r|\n/,$_);
			# check phone string
			if ( !($phone=~/^([0-9-]{3,29}|-Unpublished-)$/) && defined $sysname ) {
				push(@e, "  $point ४ ⥫䮭 - $phone");
				$phone = "-Unpublished-";
			}
			# check speed string
			my $valid=0;
			foreach $rule(@valid_speed) {
				if ($speed=~/^$rule$/) {
					$valid=1;
					last;
				}
			}
			if ( !$valid && defined $sysname) {
				push(@e, "  $point ४⭠ ᪮ - $speed");
				$speed = "300";
			}
			# check flags
			my $user=0;
			my @flags_c;
			foreach $flag (@flags) {
				my $valid=0;
				if($user==0) {
					foreach $rule(@valid_flags) {
						if ($flag eq "U") {
							push(@flags_c,$flag);
							$valid=1;
							$user=1;
							last;
						} elsif ($flag=~/^$rule$/) {
							push(@flags_c,$flag);
							$valid=1;
							last;
						}
					}
				} else {
					foreach $rule(@valid_user_flags) {
						if ($flag=~/^$rule$/) {
							push(@flags_c,$flag);
							$valid=1;
							last;
						}
					}
				}
				if ( !$valid && defined $sysname) {
					push(@e, "  $point ४ 䫠 - $flag");
				}
			}

			if (($in_format==0 || $in_format==1) && $kw=~/^Boss$/i) {
				if ($point =~ /^([0-9]{1,5}):([0-9]{1,5})\/([0-9]{1,5})$/) {
					$in_format=1;
					$z=$1;
					$n=$2;
					$f=$3;
					undef $skip_node;
					if ($node ne $f) {
						push(@e, "᪠ 訡 - ᥣ 㧫 $myzone:$mynet/$node ᮤন ଠ  㧫 $z:$n/$f");
						$critical=1;
						$skip_node=1;
					} else {
						$r.= "Boss,$point,";
						if (defined $sysname && defined $location && defined $name && defined $phone && defined $speed) {
							$r.="$sysname,$location,$name,$phone,$speed".((scalar @flag_c)?",". join(",", @flag_c):"");
						}
						$r.="\r\n";
					}
				} else {
					push(@e, "᪠ 訡 -    $point");
					$critical=1;
				}
			} elsif (($in_format==0 || $in_format==2) && $kw=~/^(Region|Host)/i) {
				if ($point>0 && $point<65536) {
					$in_format=2;
					$n=$point;
				} else {
					push(@e, "᪠ 訡 -   : $point");
					$critical=1;
				}
			} elsif ($in_format==2 && $kw eq "") {
				if ($point>0 && $point<32768) {
                			$in_format=2;
					undef $skip_node;
					$f=$point;
					$nlstring = find_node("$z:$n/$f");
					if ($node ne $f) {
						push(@e, "᪠ 訡 - ᥣ 㧫 $myzone:$mynet/$node ᮤন ଠ  㧫 $z:$n/$f");
						$critical=1;
						$skip_node=1;
					} else {
						$r.= "Boss,$z:$n/$f,";
						if (defined $sysname && defined $location && defined $name && defined $phone && defined $speed) {
							$r.="$sysname,$location,$name,$phone,$speed".((scalar @flag_c)?",". join(",", @flag_c):"");
						}
						$r.="\r\n";
					}
				} else {
					push(@e, "᪠ 訡 -   ⥬: $point");
					$critical=1;
				}
			} elsif ( $_ =~ /^[\r\n]*$/ || $skip_node ) {
				push(@e, " 窠");
#				$critical=1;
#				$e=();
			} elsif (
				($in_format==1 && ($kw eq '' || $kw=~/^point$/i)) ||
				($in_format==2 && $kw=~/^point$/i )
				) {
				if ($kw=~/^point$/i) {
					$fixed=1;
				}
				# check point number
				if ( !($point>0 && $point<32768) ) {
					push(@e, "᪠ 訡 - ⮢  $point ४⥭");
					$critical=1;
#					$point = "65536";
				} else {
					$r.=",$point,$sysname,$location,$name,$phone,$speed".((scalar @flags_c)?",". join(",", @flags_c):"") . "\r\n";
				}
			} else {
				push(@e, "᪠ 訡 - ୠ 窠: ".$_);
#				push(@e, "ୠ 窠 ᮤঠ: \"".$_."\"");
				$critical=1;
			}
		}
		if ($format) {
			$ret.=$r;
		}
		if ($error) {
			$ret.=$e;
		} 
		if (!$error && !$format && length($e)>0) {
			close(PLIST);
			return $e;
		}
	}
	close(PLIST);
	if ($fixed && @e) {
		$segstatus="*> ࠡ⠭ *>";
	} elsif ($fixed && !@e) {
		$segstatus=">> ࠡ⠭ >>";
	} elsif (!$fixed && @e) {
		$segstatus="** ࠡ⠭ **";
	} else {
		$segstatus=" ࠡ⠭ ";
	}
	return $ret;
}

@nodes=sort { $a <=> $b } @nodes;
my $report;
my $PNTLISTCNT;
$report.="Ŀ\n";
$report.="                  ⨪ 樨 ⫨  $mynet                  \n";
$report.="Ĵ\n";
$report.="            䠩 ᥣ    訡   ᥣ \n";
$report.="Ĵ\n";
foreach $node (@nodes)
{
	$file="$segpath/PNT$mynet.".addingnull($node,3);
	if (!stat($file))
	{
		$errors="?";
		$datetime_string="**  䠩 ᥣ **";
		$segstatus="x  x";
	} else {
		if (@status[$node] eq 'Normal') {
			@e = ();
			$data=check_plist($file,$mynet,$out_format,$out_error,$node).";\r\n";
			if (@e) {
				my $errormessage="ࠢ, ᮯ 㧫 $myzone:$mynet/$node!\n\n ࠡ⪥ 襣 ᥣ 뫨 㦥 ᫥騥 訡:\n";
				$count=0;
				foreach $error (@e) {
					$count=$count+1;
					$errormessage.="$count) $error\n";
				}
				if ($critical) {
					$errormessage.="\n  뫨  ᪨ 訡 -  ᥣ  㤥 ⢮  ⫨    ࠢ !\n";
				}
				if ($fixed) {
					$errormessage.="\n  ᥣ  ᮮ⢥ ଠ ஢ ⫨,  ⥯ ॡ ⥪騬 R50PK.  㤥 ⮬᪨ ஢,    , ਢ ᢮ ᥣ  ଠ쭮  ( 祪 稭  Point).\n";
				}
				$errormessage.="\n 㢠, PNT$mynet.NDL Maker  $addrfrom.\n";
				my $mail_process = $mail_program;
				$mail_process =~ s/TARGET/Netmail/g;
				$mail_process =~ s/TONAME/@sysopname[$node]/g;
				$mail_process =~ s/TOADDR/$myzone:$mynet\/$node/g;
				$subject="ந諨 訡  ࠡ⪥ ᥣ PNT$mynet.".addingnull($node,3);
				$mail_process =~ s/SUBJECT/$subject/g;
				open OUTPUT, "| $mail_process";
				print OUTPUT "$errormessage";
				close OUTPUT;
			}
			if (!$critical && $makemode) {
				$PNTLISTCNT.=$data;
			} else {
				$segstatus="++ ⪫ ++";
			}
			$errors=@e;
		} else {
			$errors="?";
			$segstatus="--- १ ---";
		}
		$datetime_string = ctime(stat($file)->mtime);
	}
	$report.=sprintf(" %-12s  %6s  %24s  %6s  %15s \n", "$myzone:$mynet/$node", @status[$node], $datetime_string, $errors, $segstatus);
}
$report.="Ĵ\n";
$report.="                       ਬ砭   ᥣ⮢                       \n";
$report.="Ĵ\n";
$report.="   ⫨   ࠡ⠭                     \n";
$report.=" >  ଠ,   ᮮ⢥⢮ ஢ ଠ, ᪮஢ \n";
$report.=" *  宩,   㦥 訡, ࠢ ⮢뢠      \n";
$report.=" -    ࠡ⠭, ⠪  㧥  䫠 Hold  Down          \n";
$report.=" +    ࠡ⠭, ⠪   ᪠ 訡              \n";
$report.=" x    ࠡ⠭, ⠪   ᠬ 䠩 ᥣ           \n";
$report.="\n";
if ($makemode) {
    if ($reportecho) {
	my $mail_process = $mail_program;
	$mail_process =~ s/TARGET/$reportecho/g;
	$mail_process =~ s/TONAME/All/g;
	$mail_process =~ s/TOADDR//g;
	$subject="⨪ 樨 ⫨  $mynet";
	$mail_process =~ s/SUBJECT/$subject/g;
	open OUTPUT, "| $mail_process";
	print OUTPUT "$report";
	close OUTPUT;
    } elsif ($reportfile) {
	open REPORT, ">$reportfile" || die "Error creating report :(\n";
	print REPORT "$report";
	close(REPORT);
    }
    ($YEARDAY) = (CORE::localtime()) [7];
    $DAY=strftime "%d", CORE::localtime();
    $WDAY=strftime "%A", CORE::localtime();
    $MONTH=strftime "%b", CORE::localtime();
    $YEAR=strftime "%Y", CORE::localtime();
    print POINTLIST ";A Net $mynet pointlist for $WDAY $MONTH $DAY $YEAR day $YEARDAY : ".addingnull(crc16($PNTLISTCNT),5)."\r\n;\r\n";
    print POINTLIST $PNTLISTCNT;
    close(POINTLIST);
}
